﻿# coding = utf-8
###################################################################################
#                                  MoistureMap                                    #
#         Toolset for creating an estimate soil moisture distribution map         #
#                                                                                 #
#                                                                                 #
#                      Created by Jan Devátý, 2020                                #
#          Department of Irrigation, Drainage and Landscape Engineering           #
#            Faculty of Civil Engineering, Czech Technical University             #
#                                                                                 #
###################################################################################

import arcpy
import os
import locale
import numpy 
import urllib

#import sys
#reload(sys)
#sys.setdefaultencoding('utf-8')

#import glob
#import shapefile
#from dbfpy import dbf

locale.setlocale(locale.LC_ALL, '')
sr = arcpy.SpatialReference(5514)
arcpy.env.overwriteOutput = True

# FILE NAMES ------------------------------------------------------------------------------------------
taskSettingsFileName = "_settings"
sourceImageFilePath = "source_image"
imageCheckZonesFileName = "check_zones"
statsFileName = "check_stats"
pointsFileName = "points"
resultImageFileName = "result_image"

# FIELD NAMES -------------------------------------------------------------------------------------------
localIDfieldName = "mID"
inputImageValueFieldName = "input_value"
resultImageValueFieldName = "model_value"
fitErrorFieldName = "fit_error"
calibrationPointsPathFieldName = "calibration_points_path"
pointsMoistureFieldName = "moisture_field_name"
calibrationImagePathFieldName = "image_path"
blurDiameterFieldName = "filter_diameter"
polynomdegreeFieldName = "polynom_degree"
polynomCoefsFieldName = "polynom_coefs"
leaveoutMaskFieldName = "mask_polygons"
tempLimitFieldName = "temperature_limit"
blurMethodFieldName = "blur_method"

# FUNCTIONS DEFINITION ------------------------------------------------------------------------------------------
# deletes files from input list
def deleteTemporaryFiles(toBeDeleted):
    if len(toBeDeleted)>0:
        arcpy.AddMessage("Mazání dočasných souborů ... ")
        #arcpy.AddMessage(str(toBeDeleted))
        for file in toBeDeleted:
            try:
                arcpy.Delete_management(file, "")
                pass
            except Exception as e:
                arcpy.AddError(e.message)
                arcpy.AddError("Mazání dočasných souborů '"+str(file)+"' selhalo!")        
        arcpy.AddMessage(" ... dokončeno.")

# returns tuple of field names
def getFieldNames(featureClass):
    fieldNames = []
    try:
        desc = arcpy.Describe(featureClass)
        fields = desc.fields
        #arcpy.ListFields(featureClass)
        for field in fields:
            fieldNames.append(field.name)
    except:
        arcpy.AddError(arcpy.GetMessages())
    return fieldNames

# checks if given extension is correct for the workspace containing the file testPath
def checkFileExtension(testPath, ext):
    dirname = os.path.dirname(testPath)
    filext = os.path.splitext(os.path.basename(testPath))
    filename = filext[0]
    extension = filext[1]

    rightPath = testPath
    if arcpy.Describe(dirname).dataType == "Folder":
        if len(filext) == 1:
            rightPath = testPath+"."+ext
        if len(filext) > 1:
            if extension != ext:
                filename = ".".join(filext[:-1])
                rightPath = os.path.join(dirname, filename)+"."+ext
    else:
        rightPath = os.path.join(dirname, filename)
    return rightPath

# function for propper formating of the SQL SELECT statement
def getSQLQuery(lst):
    if not isinstance(lst, (list, tuple)):
        if isinstance(lst, basestring):
            sql = " = '"+str(lst)+"'"
        else:
            sql = " = "+str(lst)
    else:
        sql = " IN ("
        i = 0
        for item in lst:
            if isinstance(item, basestring):
                sql += "'"+str(item)+"'"
            else:
                sql += str(item)
            if i < len(lst) - 1:
                sql += ", "
            else:
                sql += ")"
            i += 1
    return sql

def checkSpatialReference(dataset):
    if arcpy.Describe(dataset).spatialReference.factoryCode in (0, "", None):
        arcpy.DefineProjection_management (dataset, sr)

def getFieldNames(featureClass):
    fieldNames = []
    try:
        desc = arcpy.Describe(featureClass)
        fields = desc.fields
        #arcpy.ListFields(featureClass)
        for field in fields:
            fieldNames.append(field.name)
    except:
        arcpy.AddError(arcpy.GetMessages())
    return fieldNames

class Toolbox(object):
    def __init__(self):
        """This toolbox uses ESRI ArcGIS API functions t."""
        self.label = "MoistureMap"
        self.alias = ""

        # List of tool classes associated with this toolbox
        self.tools = [SetUpTask, TrasformRaster]

class SetUpTask(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "Nová úloha - kalibrace, výpočet"
        self.description = "Založí geodatbázi úlohy a importuje potřebné datasety, vypočítá kalibrační vztah a uloží jej."
        self.canRunInBackground = False
        self.category = ""

    def getParameterInfo(self):
        """Define parameter definitions"""
        taskWorkspace = arcpy.Parameter(
            displayName = "Geodatabáze úlohy",
            name = "taskWorkspace",
            datatype = "DEWorkspace",
            parameterType = "Required",
            direction = "Input"
            )
        # Feature class or Layer with points containing known moistures
        inputPoints = arcpy.Parameter(
            displayName = "Dataset bodů se známými vlhkostmi",
            name = "inputPoints",
            datatype = "GPFeatureLayer",
            parameterType = "Required",
            direction = "Input"
            )
        inputPoints.filter.list = ["Point"]
        pointIDfieldName = arcpy.Parameter(
            displayName = "Atributové pole obsahující identifikátor bodu",
            name = "pointIDfieldName",
            datatype = "Field",
            parameterType = "Optional",
            direction = "Input"
            )
        pointIDfieldName.parameterDependencies = [inputPoints.name]
        # Field containing the moisture values
        moistureFieldName = arcpy.Parameter(
            displayName = "Atributové pole obsahující hodnoty vlhkostí",
            name = "moistureFieldName",
            datatype = "Field",
            parameterType = "Required",
            direction = "Input"
            )
        moistureFieldName.parameterDependencies = [inputPoints.name]
        # thermal image raster
        inputThermalImage = arcpy.Parameter(
            displayName = "Vstupní snímek termální kamery",
            name = "inputThermalImage",
            datatype = "GPRasterLayer",
            parameterType = "Required",
            direction = "Input"
            )
        polynomDegree = arcpy.Parameter(
            displayName = "Řád polynomu pro řešení úlohy",
            name = "polynomDegree",
            datatype = "GPLong",
            parameterType = "Required",
            direction = "Input"
            )
        polynomDegree.filter.type = "ValueList"
        polynomDegree.filter.list = [1,2,3,4,5]
        polynomDegree.value = 3
        
        blurThermalImage = arcpy.Parameter(
            displayName = "Vyhladit termosnímek",
            name = "blurThermalImage",
            datatype = "GPBoolean",
            parameterType = "Optional",
            direction = "Input",
            category = "Vyhlazení termosnímku"
            )
        blurDiameter = arcpy.Parameter(
            displayName = "Průměr oblasti [m] pro vyhlazení termosnímku",
            name = "blurDiameter",
            datatype = "GPDouble",
            parameterType = "Optional",
            direction = "Input",
            category = "Vyhlazení termosnímku"
            )
       
        tempLimit = arcpy.Parameter(
            displayName = "Vynechat ze zpracování hodnoty větší než:",
            name = "tempLimit",
            datatype = "GPDouble",
            parameterType = "Optional",
            direction = "Input",
            category = "Vynechat oblasti"
            )
        tempLimit.value = 100
        # Feature class or Layer with areas to be erased from the thermal image
        leaveoutPolygons = arcpy.Parameter(
            displayName = "Polygony oblastí pro vynechání z analýzy",
            name = "leaveoutPolygons",
            datatype = "GPFeatureLayer",
            parameterType = "Optional",
            direction = "Input",
            category = "Vynechat oblasti"
            )
        leaveoutPolygons.filter.list = ["Polygon"]
        params = [taskWorkspace, inputPoints, pointIDfieldName, moistureFieldName, inputThermalImage, polynomDegree, blurThermalImage, blurDiameter, tempLimit, leaveoutPolygons]
        return params

    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal validation is performed.  This method is called whenever a parameter has been changed.#"""

        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool parameter.  This method is called after internal validation.#"""
        return

    def execute(self, parameters, messages):
        """The source code of the tool."""
        taskWorkspace = parameters[0].valueAsText
        inputPointsSource = parameters[1].valueAsText
        idFieldName = parameters[2].valueAsText
        moistureFieldName = parameters[3].valueAsText
        inputThermalImage = parameters[4].valueAsText
        polynomDegree = parameters[5].value
        blurThermalImage = parameters[6].value
        blurDiameter = parameters[7].value
        tempLimit = parameters[8].value
        leaveoutPolygons = parameters[9].valueAsText

        toBeDeleted = []
        
        # get paths to input datasets
        inputPointsDatasetPath = arcpy.Describe(inputPointsSource).catalogPath
        inputImageFilePath = arcpy.Describe(inputThermalImage).catalogPath
        
       
        # set the image raster as reference for cellsize and extent
        arcpy.env.extent = inputThermalImage
        arcpy.env.cellSize = Cell_SizeX = arcpy.GetRasterProperties_management (inputThermalImage, "CELLSIZEX").getOutput(0)

        leaveoutPolygonsFilePath = arcpy.Describe(leaveoutPolygons).catalogPath
        
# CREATE AND FILL TASK SETTINGS TABLE ----------------------------------------------------------------------------------------
        taskSettingsFilePath = taskWorkspace+os.sep+taskSettingsFileName
        #if arcpy.Exists(taskSettingsFilePath):

        try:
            arcpy.CreateTable_management (taskWorkspace, taskSettingsFileName)
            arcpy.AddField_management(taskSettingsFilePath, calibrationPointsPathFieldName, "TEXT", "", "", "", "", "", "", "")
            
            arcpy.AddField_management(taskSettingsFilePath, pointsMoistureFieldName, "TEXT", "", "", "", "", "", "", "")
            arcpy.AddField_management(taskSettingsFilePath, calibrationImagePathFieldName, "TEXT", "", "", "", "", "", "", "")
            arcpy.AddField_management(taskSettingsFilePath, blurDiameterFieldName, "DOUBLE", "", "", "", "", "", "", "")
            arcpy.AddField_management(taskSettingsFilePath, leaveoutMaskFieldName, "TEXT", "", "", "", "", "", "", "")
            arcpy.AddField_management(taskSettingsFilePath, tempLimitFieldName, "DOUBLE", "", "", "", "", "", "", "")
            arcpy.AddField_management(taskSettingsFilePath, polynomdegreeFieldName, "SHORT", "", "", "", "", "", "", "")
            arcpy.AddField_management(taskSettingsFilePath, polynomCoefsFieldName, "TEXT", "", "", "", "", "", "", "")
            with arcpy.da.InsertCursor(taskSettingsFilePath, [calibrationPointsPathFieldName, pointsMoistureFieldName, calibrationImagePathFieldName, blurDiameterFieldName, leaveoutMaskFieldName, tempLimitFieldName]) as table:
                table.insertRow([inputPointsDatasetPath, moistureFieldName, inputImageFilePath, blurDiameter, leaveoutPolygonsFilePath, tempLimit])
        except Exception as e:
            arcpy.AddMessage(e)
            arcpy.AddMessage(u"Chyba při vytváření tabulky nastavení úlohy '"+taskSettingsFilePath+"'")
        else:

            arcpy.AddMessage("Tabulka nastavení úlohy úspěšně vytvořena.")
            
        IDfield = arcpy.Describe(taskSettingsFilePath).OIDFieldName
        imageID = 0
        with arcpy.da.SearchCursor(taskSettingsFilePath, [IDfield]) as table:
            for row in table:
                imageID = min(imageID, row[0])

        
# THERMAL IMAGE PRE-PROCESSING --------------------------------------------------------------------------
        sourceImageImageFilePath = taskWorkspace+os.sep+sourceImageFilePath+"_"+str(imageID)
        if (blurThermalImage or tempLimit not in (0, "", None, False) or leaveoutPolygons not in (0, "", None, False) ):
            arcpy.AddMessage("Úprava termálního snímku ...")
            inRas = inputThermalImage
            if (tempLimit not in (0, "", None, False)):
                cropedImage = arcpy.sa.SetNull(inRas, inRas, "VALUE > "+str(tempLimit))
                cropedImage.save(taskWorkspace+os.sep+"croped_image_"+str(imageID))
                inRas = cropedImage
                workingThermalImage = cropedImage
                
            if (leaveoutPolygons not in (0, "", None, False)):
                polyid = arcpy.Describe(leaveoutPolygons).OIDFieldName
                leaveoutRaster = arcpy.PolygonToRaster_conversion(leaveoutPolygons, polyid, taskWorkspace+os.sep+"leaveout_raster_"+str(imageID), "CELL_CENTER", "", "")
                toBeDeleted.append(leaveoutRaster)
                maskedImage = arcpy.sa.Con(arcpy.sa.IsNull(leaveoutRaster), inRas, None)
                maskedImage.save(taskWorkspace+os.sep+"masked_image_"+str(imageID))
                inRas = maskedImage
                workingThermalImage = maskedImage
                #
                
            if (blurThermalImage) :
                if (blurDiameter in (0, "", None, False)):
                    arcpy.AddWarning("Hodnota velikosti oblasti pro průměrování nebyla zadána!")
                else:
                    workingThermalImage = arcpy.sa.FocalStatistics (inRas, arcpy.sa.NbrCircle(blurDiameter/2, "MAP"), "MEAN", "DATA")
                    workingThermalImage.save(sourceImageImageFilePath)
            
            arcpy.AddMessage(" ... dokončeno.")
        else:
            workingThermalImage = arcpy.sa.Raster(inputThermalImage)

# POINT ANALYSIS -----------------------------------------------------------------------------------------
        pointsFilePath = taskWorkspace+os.sep+pointsFileName
        points = arcpy.CopyFeatures_management (inputPointsSource, pointsFilePath)
       
        if (idFieldName in (None, 0, "", False)):
            idFieldName = localIDfieldName
            if (localIDfieldName not in getFieldNames(points)):
                arcpy.AddField_management(points, localIDfieldName, "SHORT", "", "", "", "", "", "", "")
                i = 1
                with arcpy.da.UpdateCursor(points, [localIDfieldName]) as table:
                    for row in table:
                        row[0] = i
                        table.updateRow(row)
                        i += 1
            idFieldName = localIDfieldName
        
        
        if (blurThermalImage and blurDiameter not in (0, "", None, False)) :
            arcpy.AddMessage("Analýza použitelnosti měřených bodů ...")
            # vytvořit kolečka pro statistiku
            checkZonesFilePath = taskWorkspace+os.sep+imageCheckZonesFileName
            checkZones = arcpy.Buffer_analysis(points, checkZonesFilePath, blurDiameter/2)
            toBeDeleted.append(checkZones)

            # zonalStats
            statsTableFilePath = taskWorkspace+os.sep+statsFileName

            if arcpy.sa.ZonalStatisticsAsTable (checkZones, idFieldName, inputThermalImage, statsTableFilePath, "DATA", "ALL"):
                toBeDeleted.append(statsTableFilePath)

                arcpy.JoinField_management (points, idFieldName, statsTableFilePath, idFieldName, ["AREA", "MEAN", "MAX", "MIN"])
            else:
                arcpy.AddWarning("Chyba při výpočtu statistik na zóně bodu.")

            # odstranění bodů, které jsou ovlivněny rozsahem thermálního snímku

            with arcpy.da.UpdateCursor(points, ["MEAN", "MAX", "MIN", idFieldName]) as table:
                for row in table:
                    if ((row[0] is None or row[1] is None or row[2] is None) or (max(row[0], row[1], row[2])>100)):
                        table.deleteRow()
                        arcpy.AddMessage("     Pro bod měření '"+str(row[3])+"' není v termálním snímku dost informací a byl odstraněn z analýzy.")

            arcpy.AddMessage(" ... dokončeno.")

        # interpolace hodnot snímku do bodů
        arcpy.AddMessage("Interpolace hodnot termálního snímku ...")
        pointsValueFileName= "points_value"
        pointsValueFilePath = taskWorkspace+os.sep+pointsValueFileName
        arcpy.sa.ExtractValuesToPoints (points, workingThermalImage, pointsValueFilePath, "INTERPOLATE", "VALUE_ONLY")
        toBeDeleted.append(pointsValueFilePath)
        arcpy.JoinField_management (points, idFieldName, pointsValueFilePath, idFieldName, ["RASTERVALU"])
        thermalValueFieldName = "RASTERVALU"
        arcpy.AddField_management(points, inputImageValueFieldName, "DOUBLE", "", "", "", "", "", "", "")
        
        with arcpy.da.UpdateCursor(points, ["RASTERVALU", idFieldName, inputImageValueFieldName]) as table:
            for row in table:
                if ((row[0] is None) or (row[0]>100)):
                    table.deleteRow()
                    arcpy.AddMessage("     Bod měření '"+str(row[1])+"' má neplatnou hodnotu ("+str(row[0])+") a byl odstraněn z analýzy.")
                else:
                    row[2] = row[0]
                    table.updateRow(row)
        
        arcpy.DeleteField_management(points, "RASTERVALU")
        
        arcpy.AddMessage(" ... dokončeno.")

# REGRESSION SOLUTION -------------------------------------------------------------------------------------------
        arcpy.AddMessage("Výpočet regresního polynomu ...")
        measuredData = []
        with arcpy.da.SearchCursor(points, [moistureFieldName]) as table:
            for row in table:
                measuredData.append(row[0])
                
        thermalData = []
        with arcpy.da.SearchCursor(points, [inputImageValueFieldName]) as table:
            for row in table:
                thermalData.append(row[0])
        
        #arcpy.AddMessage(str(measuredData))
        #arcpy.AddMessage(str(thermalData))
        
        poly = numpy.polyfit(thermalData, measuredData, polynomDegree)
        #arcpy.AddMessage(str(poly))
        coefstring = ""
        i = 0
        for coef in poly:
            if i != 0:
                coefstring += ";"
            coefstring += str(coef)
            i += 1
        
        i = 0
        with arcpy.da.UpdateCursor(taskSettingsFilePath, [polynomdegreeFieldName, polynomCoefsFieldName]) as table:
            for row in table:
                if i == 0:
                    row[0] = polynomDegree
                    row[1] = coefstring
                    table.updateRow(row)
                i += 1
                
        arcpy.AddMessage(" ... dokončeno.")

# THERMAL IMAGE TRANSFORMATION --------------------------------------------------------------------------
        arcpy.AddMessage("Transformace vstupního snímku ...")
        polypredict = numpy.poly1d(poly)
        
        #inputRaster = arcpy.sa.Raster(workingThermalImage)
        resultImagePath = taskWorkspace+os.sep+resultImageFileName+"_"+str(imageID)
        outputRasterPoly = polypredict(workingThermalImage)
        outputRasterPoly.save(resultImagePath)
        arcpy.AddMessage(" ... dokončeno.")

# ERROR CALCULATION --------------------------------------------------------------------------        
        arcpy.AddMessage("Výpočet chyb modelu ...")
        pointsIntValueFileName= "points_interp_value"
        pointsIntValueFilePath = taskWorkspace+os.sep+pointsValueFileName
        arcpy.sa.ExtractValuesToPoints (points, resultImagePath, pointsIntValueFilePath, "INTERPOLATE", "VALUE_ONLY")
        toBeDeleted.append(pointsIntValueFilePath)
        
        interpolatedValues = {}
        with arcpy.da.SearchCursor(pointsIntValueFilePath, [idFieldName, "RASTERVALU"]) as table:
            for row in table:
                interpolatedValues.update({row[0]:row[1]})
        #arcpy.AddMessage(str(interpolatedValues))
        
        arcpy.AddField_management(points, resultImageValueFieldName, "DOUBLE", "", "", "", "", "", "", "")
        arcpy.AddField_management(points, fitErrorFieldName, "DOUBLE", "", "", "", "", "", "", "")
        with arcpy.da.UpdateCursor(points, [idFieldName, moistureFieldName, resultImageValueFieldName, fitErrorFieldName]) as table:
            for row in table:
                row[2] = interpolatedValues[row[0]]
                row[3] = interpolatedValues[row[0]]-row[1]
                table.updateRow(row)
        
        arcpy.AddMessage(" ... dokončeno.")
        
        deleteTemporaryFiles(toBeDeleted)

# ADD THE LAYERS TO MAP AND APPLY SYMBOLOGY -----------------------------------------------------------------------------------
        mxd = arcpy.mapping.MapDocument("CURRENT")
        # get the data frame
        df = arcpy.mapping.ListDataFrames(mxd,"*")[0]
        #df.referenceScale = 5000
        newlayer = arcpy.mapping.Layer(resultImagePath)
        newlayer.name = "odhad vlhkosti"
        moistureLayerSymbology = sys.path[0]+os.sep+"moisturemap.lyr"
        arcpy.ApplySymbologyFromLayer_management(newlayer, moistureLayerSymbology)
        #newlayer.transparency = transp
        arcpy.mapping.AddLayer(df, newlayer, "TOP")
        
        newlayer = arcpy.mapping.Layer(pointsFilePath)
        newlayer.name = "použité body"
        pointsLayerSymbology = sys.path[0]+os.sep+"points.lyr"
        #arcpy.mapping.UpdateLayer (df, newlayer, pointsLayerSymbology, False)
        arcpy.ApplySymbologyFromLayer_management(newlayer, pointsLayerSymbology)
        
        arcpy.mapping.AddLayer(df, newlayer, "TOP")
        
        return

class TrasformRaster(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "Transformovat termální snímek dle kalibrace"
        self.description = "Transformuje zvolený snímek podle dříve provedené kalibrace"
        self.canRunInBackground = False
        self.category = ""

    def getParameterInfo(self):
        """Define parameter definitions"""
        taskWorkspace = arcpy.Parameter(
            displayName = "Geodatabáze úlohy",
            name = "taskWorkspace",
            datatype = "DEWorkspace",
            parameterType = "Required",
            direction = "Input"
            )
        # thermal image raster
        inputThermalImage = arcpy.Parameter(
            displayName = "Vstupní snímek termální kamery",
            name = "inputThermalImage",
            datatype = "GPRasterLayer",
            parameterType = "Required",
            direction = "Input"
            )
        tempLimit = arcpy.Parameter(
            displayName = "Vynechat ze zpracování hodnoty větší než:",
            name = "tempLimit",
            datatype = "GPDouble",
            parameterType = "Optional",
            direction = "Input",
            category = "Vynechat oblasti"
            )
        tempLimit.value = 100
        # Feature class or Layer with areas to be erased from the thermal image
        leaveoutPolygons = arcpy.Parameter(
            displayName = "Polygony oblastí pro vynechání z analýzy",
            name = "leaveoutPolygons",
            datatype = "GPFeatureLayer",
            parameterType = "Optional",
            direction = "Input",
            category = "Vynechat oblasti"
            )
        leaveoutPolygons.filter.list = ["Polygon"]
        params = [taskWorkspace, inputThermalImage, tempLimit, leaveoutPolygons]
        return params

    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal validation is performed.  This method is called whenever a parameter has been changed.#"""

        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool parameter.  This method is called after internal validation.#"""
        return

    def execute(self, parameters, messages):
        """The source code of the tool."""
        taskWorkspace = parameters[0].valueAsText
        inputThermalImage = parameters[1].valueAsText
        tempLimit = parameters[2].value
        leaveoutPolygons = parameters[3].valueAsText
        
        toBeDeleted = []
        
        # set the image raster as reference for cellsize and extent
        arcpy.env.extent = inputThermalImage
        arcpy.env.cellSize = Cell_SizeX = arcpy.GetRasterProperties_management (inputThermalImage, "CELLSIZEX").getOutput(0)
        
        leaveoutPolygonsFilePath = arcpy.Describe(leaveoutPolygons).catalogPath
        inputImageFilePath = arcpy.Describe(inputThermalImage).catalogPath
        
# LOAD THE SETTINGS FROM SETTINGS TABLE ----------------------------------------------------------------
        taskSettingsFilePath = taskWorkspace+os.sep+taskSettingsFileName
        if (not arcpy.Exists(taskSettingsFilePath)):
            arcpy.AddError(u"Tabulka s nastavením úlohy neexistuje!")
            arcpy.AddError(u"Nejprve proveďte kalibraci.")
            return
        else:
            IDfield = arcpy.Describe(taskSettingsFilePath).OIDFieldName
            lastID = 0
            polyDegree = 0
            polyCoefsString = ""
            blurDiameter = 0
            i = 1
            with arcpy.da.SearchCursor(taskSettingsFilePath, [IDfield, polynomdegreeFieldName, polynomCoefsFieldName, blurDiameterFieldName, blurMethodFieldName]) as table:
                for row in table:
                    if i == 1:
                        polyDegree = row[1]
                        polyCoefsString = row[2]
                        blurDiameter = row[3]
                    lastID = max(lastID, row[0])
                    i += 1
            poly = []
            for coefstr in polyCoefsString.split(";"):
                try:
                    poly.append(float(coefstr))
                except Exception as e:
                    arcpy.AddError(e.message)
                    arcpy.AddError("Načítání koeficientů polynomu selhalo!")
                    return
            if (polyDegree+1 != len(poly)):
                arcpy.AddError("Udaný stupeň polynomu ("+str(polyDegree)+") nesouhlasí s počtem koeficientů ("+str(len(poly))+")")   
                return

        newID = lastID+1

# THERMAL IMAGE PRE-PROCESSING --------------------------------------------------------------------------
        
        if blurDiameter in (0, "", None, False):
            blurThermalImage = False
        else:
            blurThermalImage = True
            
        sourceImageImageFilePath = taskWorkspace+os.sep+sourceImageFilePath+"_"+str(newID)
        if (blurThermalImage or tempLimit not in (0, "", None, False) or leaveoutPolygons not in (0, "", None, False) ):
            arcpy.AddMessage("Úprava termálního snímku ...")
            inRas = inputThermalImage
            if (tempLimit not in (0, "", None, False)):
                cropedImage = arcpy.sa.SetNull(inRas, inRas, "VALUE > "+str(tempLimit))
                cropedImage.save(taskWorkspace+os.sep+"croped_image_"+str(newID))
                inRas = cropedImage
                workingThermalImage = cropedImage
                
            if (leaveoutPolygons not in (0, "", None, False)):
                polyid = arcpy.Describe(leaveoutPolygons).OIDFieldName
                leaveoutRaster = arcpy.PolygonToRaster_conversion(leaveoutPolygons, polyid, taskWorkspace+os.sep+"leaveout_raster_"+str(newID), "CELL_CENTER", "", "")
                toBeDeleted.append(leaveoutRaster)
                maskedImage = arcpy.sa.Con(arcpy.sa.IsNull(leaveoutRaster), inRas, None)
                maskedImage.save(taskWorkspace+os.sep+"masked_image_"+str(newID))
                inRas = maskedImage
                workingThermalImage = maskedImage
                #
                
            if (blurThermalImage) :
                if (blurDiameter in (0, "", None, False)):
                    arcpy.AddWarning("Hodnota velikosti oblasti pro průměrování nebyla zadána!")
                else:
                    workingThermalImage = arcpy.sa.FocalStatistics (inRas, arcpy.sa.NbrCircle(blurDiameter/2, "MAP"), "MEAN", "DATA")
                    workingThermalImage.save(sourceImageImageFilePath)
            
            arcpy.AddMessage(" ... dokončeno.")
        else:
            workingThermalImage = arcpy.sa.Raster(inputThermalImage)

# THERMAL IMAGE TRANSFORMATION --------------------------------------------------------------------------
        arcpy.AddMessage("Transformace vstupního snímku ...")
        polypredict = numpy.poly1d(poly)
        
        resultImagePath = taskWorkspace+os.sep+resultImageFileName+"_"+str(newID)
        outputRasterPoly = polypredict(workingThermalImage)
        outputRasterPoly.save(resultImagePath)
        arcpy.AddMessage(" ... dokončeno.")
        
        deleteTemporaryFiles(toBeDeleted)
        
        with arcpy.da.InsertCursor(taskSettingsFilePath, [calibrationImagePathFieldName, leaveoutMaskFieldName]) as table:
            table.insertRow([inputImageFilePath, leaveoutPolygonsFilePath])
                
                
# ADD THE LAYERS TO MAP AND APPLY SYMBOLOGY -----------------------------------------------------------------------------------
        mxd = arcpy.mapping.MapDocument("CURRENT")
        # get the data frame
        df = arcpy.mapping.ListDataFrames(mxd,"*")[0]
        #df.referenceScale = 5000
        newlayer = arcpy.mapping.Layer(resultImagePath)
        newlayer.name = "odhad vlhkosti "+str(newID)
        moistureLayerSymbology = sys.path[0]+os.sep+"moistureLayer.lyr"
        arcpy.ApplySymbologyFromLayer_management(newlayer, moistureLayerSymbology)
        #newlayer.transparency = transp
        arcpy.mapping.AddLayer(df, newlayer, "TOP")
        
        return
    